#!/usr/local/bin/perl -w

## This file contains subroutines that are used in the Rweb statistical
## analysis package. It should be available in the main Rweb script 
## directory.

########################################################################
### ReadParse
### Reads and parses the values returned by a Web browser when calling
### a cgi script ... Everything is put into the %in hash
########################################################################
           # Adapted from cgi-lib.pl by S.E.Brenner@bioc.cam.ac.uk
           # Copyright 1994 Steven E. Brenner
           # modified by Jeff Banfield 12/18/97
           sub ReadParse {
             local (*in) = @_ if @_;
             local ($i, $key, $val);

             if ( $ENV{'REQUEST_METHOD'} eq "GET" ) {
               $in = $ENV{'QUERY_STRING'};
             } elsif ($ENV{'REQUEST_METHOD'} eq "POST") {
               read(STDIN,$in,$ENV{'CONTENT_LENGTH'});
             } else {
                   # Added for command line debugging
                   # Supply name/value form data as a command line argument
                   # Format: name1=value1\&name2=value2\&...
                   # (need to escape & for shell)
                   # Find the first argument that's not a switch (-)
                   $in = ( grep( !/^-/, @ARGV )) [0];
                   $in =~ s/\\&/&/g;
             }

             @in = split(/&/,$in);

             foreach $i (0 .. $#in) {
               # Convert plus's to spaces
               $in[$i] =~ s/\+/ /g;

               # Split into key and value.
               ($key, $val) = split(/=/,$in[$i],2); # splits on the first =.

               # Remove the carriage returns from Netscape ... JDB 12/18/97
               $val =~ s/%0D//ge;

               # Convert %XX from hex numbers to alphanumeric
               $key =~ s/%(..)/pack("c",hex($1))/ge;
               $val =~ s/%(..)/pack("c",hex($1))/ge;

               # Associate key and value. \0 is the multiple separator
               $in{$key} .= ", " if (defined($in{$key}));
               $in{$key} .= $val;
             }
             return length($in);
           }

###############################################################
## inputStuff
## Get the data file and input the relevant information:
## variables .... the variable names
## code ... the R code necessary to load and set up the variables
## description ... a description of the data set
## All of this is stored in the %input hash
###############################################################
sub inputStuff {

## This test is so we can give the subroutine a filename as an argument
## or, if we don't, just let it find the file name in the %in hash

	if (@_[0]) {
		$dataname = @_[0];
		}
	else{
		$dataname = $in{'datainputfile'};	# This may be a temporary file!
		}
		
	open(inputFile, "$dataname");
	@fileContents = <inputFile>;
	close(inputFile);
	
	
	# convert to scalar so we can do lots of stuff with it ...
	foreach $line (@fileContents){
	    $fileContents .= $line;
	}
	
	
	# strip returns after & and : ... this way the input file can be more readable
	$fileContents =~ s/&\n/&/g;
	$fileContents =~ s/:\n/:/g;
	
	@input = split(/&/, $fileContents);
	
	
	# Split into key and value.
	foreach $i (0 .. $#input) {
	    ($key, $value) = split(/:/,$input[$i],2); # splits on the first :	
	    $input{$key} = $value;
	}
	return $input{'description'};   # only used in the buildModule.cgi file
}

###############################################################
## Security
## remove several system calls from the input ... 
## this could be a security problem
###############################################################

sub Security {

if ($Rcommands =~ /system\s*\(.*?\)/ || 
    $Rcommands =~ /-\s*system/ ||
    $Rcommands =~ /_\s*system/){
    print "Sorry, you may not use the system function\n";
    die;
}
if ($Rcommands =~ /scan\s*\(.*?\)/ || 
    $Rcommands =~ /-\s*scan/ ||
    $Rcommands =~ /_\s*scan/){
    print "Sorry, you may not use the scan function\n";
    die;
}
if ($Rcommands =~ /read\.table\s*\(.*?\)/ || 
    $Rcommands =~ /-\s*read\.table/ ||
    $Rcommands =~ /_\s*read\.table/){
    print "Sorry, you may not use the read.table function\n";
    die;
}
if ($Rcommands =~ /count\.fields\s*\(.*?\)/ || 
    $Rcommands =~ /-\s*count\.fields/ ||
    $Rcommands =~ /_\s*count\.fields/){
    print "Sorry, you may not use the count.field function\n";
    die;
}
if ($Rcommands =~ /write\s*\(.*?\)/ || 
    $Rcommands =~ /-\s*write/ ||
    $Rcommands =~ /_\s*write/){
    print "Sorry, you may not use the write function\n";
    die;
}
if ($Rcommands =~ /sink\s*\(.*?\)/ || 
    $Rcommands =~ /-\s*sink/ ||
    $Rcommands =~ /_\s*sink/){
    print "Sorry, you may not use the sink function\n";
    die;
}
if ($Rcommands =~ /call\s*\(.*?\)/ || 
    $Rcommands =~ /-\s*call/ ||
    $Rcommands =~ /_\s*call/){
    print "Sorry, you may not use the call function\n";
    die;
}
if ($Rcommands =~ /eval\s*\(.*?\)/ || 
    $Rcommands =~ /-\s*eval/ ||
    $Rcommands =~ /_\s*eval/){
    print "Sorry, you may not use the eval function\n";
    die;
}if ($Rcommands =~ /assign\s*\(.*?\)/ || 
    $Rcommands =~ /-\s*assign/ ||
    $Rcommands =~ /_\s*assign/){
    print "Sorry, you may not use the assign function\n";
    die;
}


# $Rcommands =~ s/scan\(.*?\)/You may not use scan calls/g;
# $Rcommands =~ s/read\.table\(.*?\)/You may not use read.table calls/g;
# $Rcommands =~ s/count\.fields\(.*?\)/You may not use count.fields calls/g;
# $Rcommands =~ s/write\(.*?\)/You may not use write calls/g;
# $Rcommands =~ s/sink\(.*?\)/You may not use sink calls/g;
}

###############################################################
## bringInData
## Checks to see if there is URL or User supplied data then
## creates the necessary data files and R code to read it in
###############################################################


sub bringInData {

# dataname will be URLdata if you have a URL
# $in{'Data'} will contain the URL
# dataname will be Userdata if you have a user supplied dataset
# $in{'UserSuppliedData'} will contain the dataset

# $in will contain the name of the %in hash from the calling script

my $in = @_;
my $dataname = $in{'dataname'};

if (($dataname eq "URLdata") || ($dataname eq "Userdata")) {

    $dataFile = $FTempData;
    $dataInput = $FTempInput; 
	
	if ($dataname eq "URLdata") {
		use lib "$LOCALPERLLIB";
		use LWP::Simple;
		$dataURL = $in{'Data'};
		if (head($dataURL) ){
		    getstore ($dataURL, $dataFile);  # download the data
		    }
		else {
		  print "<H2>Something wrong with data entry</H2> \n";
		  print "There was and error trying to read <BR> <B>$dataURL</B> <BR> \n";
		  print "Please make sure the URL is correct.";
		  return(0);
    		}
    	}
    if ($dataname eq "Userdata") {
    	open(OUTFILE, "> $dataFile");
    	print OUTFILE "$in{'UserSuppliedData'}";
		close(OUTFILE);
		}
		
# create the input file so we don't have to modify the rest of the program
    open(INFILE, $dataFile);
    $line = <INFILE> || die "couldn't open the file"; # Read the variable names
    close(INFILE);

    $line =~ s/\s+/, /g;   # insert commas
    $line =~ s/, $//;        # remove the last comma

# write variable names and code to input file
    open(OUTFILE, "> $dataInput");
    print OUTFILE "variables:\n$line\n&\ncode:\n";
    print OUTFILE "X <- read.table(\"$dataFile\", header=T)\n";
    print OUTFILE "attach(X)\n";
    print OUTFILE "names(X)\n \n";
    close(OUTFILE);
}
else {
$dataInput = "$RWEBDATA/$dataname.input"; 
}
return $dataInput;
}

###############################################################
## runR
## This is just the command to run R in batch mode in the appropriate
## directory and with the correct input and output files.
###############################################################
sub runR {
	system("cd $RPROFILEDIR; $RDIR --no-save < $FCode > $FOutputTxt");
	}

###############################################################
## RwebExit
## Be nice and clean up all the temporary files
###############################################################
sub RwebExit {
  system("rm $FWarnings");
  system("rm $FOutputPS");
  system("rm $FCode");
  system("rm $FOutputTxt");
  system("rm $FOutputSink");
  system("rm $FTempData");                     # Created in the bringInData subroutine
  system("rm $FTempInput");
}

###############################################################
## createTempFiles
## Make all of the temporary files for storing code, data, ...
###############################################################
sub createTempFiles {
	$FWarnings = "$TMPDIR/Rwarning.$identifier.txt";
	$FOutputPS = "$TMPDIR/Rout.$identifier.ps";
	$FOutputPSName = "Rout.$identifier.ps";
  	$FCode = "$TMPDIR/Rcode.$identifier.txt";
  	$FOutputTxt = "$TMPDIR/Rout.$identifier.txt";
  	$FOutputSink = "$TMPDIR/Rout.$identifier.sink";
  	$FTempData = "$TMPDIR/Rdata.$identifier.data";
  	$FTempInput = "$TMPDIR/Rdata.$identifier.input";
  	$FIdentifier = $identifier;		# This is needed for some of the postscript conversions
}

###############################################################
## ps2gif
## Convert the postscript file to a series of gif files
###############################################################

sub ps2gif {
chdir($TMPDIR);
system("$PSTOIMG -g -m -flip r270 -transparent $FOutputPSName > $FWarnings");
}

###############################################################
## processPostscript
## Process the postscript file, create gif files and write the necessary code
## into a new Rgif* html file ... this all happens in the $TMPDIR directory
###############################################################

sub processPostscript{
  # Convert the postscript files to gif format
  &ps2gif;

  # what gif files do we have?
  @gifFiles = glob("Rout$FIdentifier*.gif");


  # Check to see if there are any images
  if (defined($gifFiles[0])){

    # Dump the image files
    # This uses the nph-imageDump.pl script that deletes the image files after loading
    print "<HR><BR><H2> Images </H2><BR><HR>";

    foreach $name (@gifFiles){
	print ("<IMG  SRC=\"http://$SERVER/$CGIDIR/nph-imageDump.pl?/tmp/$name\">\n");
	print ("<HR>");
    }
    print "</BODY>";

  }
}

###############################################################
## printResults
## Just opens the output file and prints the results to standard output with
## carriage returns.
###############################################################

sub printResults {
  select(STDOUT);

  print("<PRE>");

  open(ResultFile, $FOutputTxt);
  while (<ResultFile>){
    chomp();
    print ("$_ \n");
  }
  close(ResultFile);
  
  print("</PRE>\n");
}



# you need this last "1" so the file returns true!
1
   
